import json

def gen_card_to_person(title: str, message: str, email: str,) -> str:
    """
    Generates a JSON payload for a message to send to a Microsoft Teams Channel with a person tagged.
    
    Args:
        title (str): The title of the message.
        message (str): The message to send.
        email (str): The email of the person to tag.
    """
    card_to_person = {
        "type": "message",
        "attachments": [
            {
                "contentType": "application/vnd.microsoft.card.adaptive",
                "content": {
                    "type": "AdaptiveCard",
                    "body": [
                        {

                            "type": "Container",
                            "items": [
                                {
                                    "type": "TextBlock",
                                    "text": "",
                                    "wrap": "true",
                                    "size": "Large",
                                    "weight": "Bolder",
                                    "color": "Attention",
                                    "maxLines": 0,
                                    "horizontalAlignment": "Left",
                                    "id": "title"
                                },
                                {
                                    "type": "ColumnSet",
                                    "minHeight": "0px",
                                    "style": "emphasis",
                                    "columns": [
                                        {
                                            "type": "Column",
                                            "width": "stretch",
                                            "items": [
                                                {
                                                    "type": "TextBlock",
                                                    "text": "Attention:",
                                                    "wrap": "true",
                                                    "id": "lbl_attention",
                                                    "size": "Medium",
                                                    "weight": "Bolder",
                                                    "color": "Warning"
                                                },
                                                {
                                                    "type": "TextBlock",
                                                    "text": "Message: ",
                                                    "wrap": "true",
                                                    "size": "Medium",
                                                    "weight": "Bolder",
                                                    "color": "Accent",
                                                    "id": "lbl_msg"
                                                }
                                            ]
                                        },
                                        {
                                            "type": "Column",
                                            "width": "stretch",
                                            "items": [
                                                {
                                                    "type": "TextBlock",
                                                    "text": "<at>User_1</at>",
                                                    "wrap": "true",
                                                    "id": "txt_attention"
                                                },
                                                {
                                                    "type": "TextBlock",
                                                    "text": "",
                                                    "wrap": "true",
                                                    "id": "txt_msg"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ],
                    "msteams": {
                        "entities": [
                            {
                                "type": "mention",
                                "text": "<at>User_1</at>",
                                "mentioned": {
                                    "id": ""
                                }
                            }
                        ]
                    }
                }
            }
        ]
    }
    card_to_person["attachments"][0]["content"]["body"][0]["items"][0]["text"] = title
    card_to_person["attachments"][0]["content"]["body"][0]["items"][1]["columns"][1]["items"][1]["text"] = message
    card_to_person["attachments"][0]["content"]["msteams"]["entities"][0]["mentioned"]["id"] = email
    payload = json.dumps(card_to_person)
    
    return payload

def gen_card_no_person(title: str, message: str) -> str:
    """
    Generates a JSON payload for a message to send to a Microsoft Teams Channel without tagging a person.
    
    Args:
        title (str): The title of the message.
        message (str): The message to send.
    """
    card_no_person = {
        "type": "message",
        "attachments": [
            {
                "contentType": "application/vnd.microsoft.card.adaptive",
                "content": {
                    "type": "AdaptiveCard",
                    "body": [
                        {

                            "type": "Container",
                            "items": [
                                {
                                    "type": "TextBlock",
                                    "text": "",
                                    "wrap": "true",
                                    "size": "Large",
                                    "weight": "Bolder",
                                    "color": "Attention",
                                    "maxLines": 0,
                                    "horizontalAlignment": "Left",
                                    "id": "title"
                                },
                                {
                                    "type": "ColumnSet",
                                    "minHeight": "0px",
                                    "style": "emphasis",
                                    "columns": [
                                        {
                                            "type": "Column",
                                            "width": "stretch",
                                            "items": [
                                                {
                                                    "type": "TextBlock",
                                                    "text": "Message: ",
                                                    "wrap": "true",
                                                    "size": "Medium",
                                                    "weight": "Bolder",
                                                    "color": "Accent",
                                                    "id": "lbl_msg"
                                                }
                                            ]
                                        },
                                        {
                                            "type": "Column",
                                            "width": "stretch",
                                            "items": [
                                                {
                                                    "type": "TextBlock",
                                                    "text": "",
                                                    "wrap": "true",
                                                    "id": "txt_msg"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            }
        ]
    }
    card_no_person["attachments"][0]["content"]["body"][0]["items"][0]["text"] = title
    card_no_person["attachments"][0]["content"]["body"][0]["items"][1]["columns"][1]["items"][0]["text"] = message
    payload = json.dumps(card_no_person)
    return payload




"""
A simple module to simplify sending messages to Microsoft Teams Channels. 

It includes two styles of messages: one with just a message and a title, and another with a message, title, and a person to tag.
"""

import requests
from .templates import gen_card_to_person, gen_card_no_person
from typing import Optional

class TeamsAlert:
    """
    A class to send messages to Microsoft Teams Channels.

    Attributes:
        url (str): The webhook URL for the Teams Channel.
    """
    def __init__(self, url: str):
        self.url = url

    def send(self, title: str = "A Nice Title", message: str = "This is a message.", attention: Optional[str] = None) -> requests.models.Response:
        """
        Sends a message to the Teams Channel. If a person is tagged, the message will include the person's name.

        Args:
            title (str): The title of the message.
            message (str): The message to send.
            attention (Optional[str], optional): The email of the person to tag in the message.
        """
        self.title = title
        self.attention = attention
        self.message = message

        if self.attention:
            return self._send_message_with_attention()
        else:
            return self._send_message()

    def _send_message(self) -> requests.models.Response:
        """
        Sends a message to the Teams Channel without tagging a person.
        """
        payload = gen_card_no_person(self.title, self.message)
        headers = {
            'Content-Type': 'application/json'        
        }

        return requests.request("POST", self.url, headers=headers, data=payload)

    def _send_message_with_attention(self) -> requests.models.Response:
        """
        Sends a message to the Teams Channel with a person tagged.
        """
        payload = gen_card_to_person(self.title, self.message, self.attention)
        headers = {
            'Content-Type': 'application/json'        
        }

        return requests.request("POST", self.url, headers=headers, data=payload)

        


import unittest
from unittest.mock import patch, MagicMock
from teams_alert.send_card import TeamsAlert
from teams_alert.templates import gen_card_no_person, gen_card_to_person
import json

class TestTeamsAlert(unittest.TestCase):
    def setUp(self):
        self.url = "https://example.com/webhook"
        self.teams_alert = TeamsAlert(self.url)

    def test_init(self):
        self.assertEqual(self.teams_alert.url, self.url)

    @patch('teams_alert.send_card.requests.request')
    def test_send_message(self, mock_request):

        mock_response = MagicMock(status_code=200)
        mock_request.return_value = mock_response

        self.teams_alert.title = "Test Title"
        self.teams_alert.message = "Test Message"

        response = self.teams_alert._send_message()

        self.assertEqual(response.status_code, 200)

        self.assertEqual(mock_request.call_args[0][0], "POST")
        self.assertEqual(mock_request.call_args[0][1], self.url)
        self.assertEqual(mock_request.call_args[1]['headers'], {'Content-Type': 'application/json'})

        expected_payload = gen_card_no_person("Test Title", "Test Message")
        self.assert_json_equal(mock_request.call_args[1]['data'], expected_payload)

    def assert_json_equal(self, json1, json2):
        """Assert that two JSON objects are equal."""
        self.assertEqual(json.loads(json1) if isinstance(json1, str) else json1,
                         json.loads(json2) if isinstance(json2, str) else json2)

    @patch('teams_alert.send_card.requests.request')
    def test_send_message_with_attention(self, mock_request):
        mock_response = MagicMock(status_code=200)
        mock_request.return_value = mock_response

        self.teams_alert.title = "Test Title"
        self.teams_alert.message = "Test Message"
        self.teams_alert.attention = "user@example.com"

        response = self.teams_alert._send_message_with_attention()

        self.assertEqual(response.status_code, 200)

        self.assertEqual(mock_request.call_args[0][0], "POST")
        self.assertEqual(mock_request.call_args[0][1], self.url)
        self.assertEqual(mock_request.call_args[1]['headers'], {'Content-Type': 'application/json'})

        expected_payload = gen_card_to_person("Test Title", "Test Message", "user@example.com")

        self.assert_json_equal(mock_request.call_args[1]['data'], expected_payload)

if __name__ == '__main__':
    unittest.main()